@@ -35,6 +35,10 @@
 #include <gdk/gdkx.h>
 #endif
 
+#ifdef ENABLE_WAYLAND
+#include <gdk/gdkwayland.h>
+#endif
+
 #include <libxfce4util/libxfce4util.h>
 #include <libxfce4ui/libxfce4ui.h>
 #include <xfconf/xfconf.h>
@@ -57,6 +61,7 @@
 #include "xfce-notify-window.h"
 #include "xfce-notify-marshal.h"
 
+#define NOTIFICATIONS_SPEC_VERSION "1.2"
 #define SPACE 16
 #define XND_N_MONITORS xfce_notify_daemon_get_n_monitors_quark()
 
@@ -67,6 +72,8 @@ struct _XfceNotifyDaemon
     XfceNotifyGBus *xfce_iface_skeleton;
     XfceNotifyOldGBus *xfce_old_iface_skeleton;
 
+    gboolean min_width_enabled;
+    guint min_width;
     gboolean expire_timeout_enabled;
     gint expire_timeout;
     gboolean expire_timeout_allow_override;
@@ -115,6 +122,11 @@ typedef struct
     XfceNotifyFdoGBusSkeletonClass parent_class;
 } XfceNotifyDaemonClass;
 
+typedef struct {
+    XfceNotifyDaemon *xndaemon;
+    XfceNotifyWindow *window;
+} MonitorCheckData;
+
 // This deliberately leaves out '/theme', as that one is handled in
 // a special way.
 static struct {
@@ -129,6 +141,18 @@ static struct {
         const gchar *s;
     } default_value;
 } settings[] = {
+    {
+        .name = MIN_WIDTH_ENABLED_PROP,
+        .type = G_TYPE_BOOLEAN,
+        .offset = G_STRUCT_OFFSET(XfceNotifyDaemon, min_width_enabled),
+        .default_value.b = FALSE,
+    },
+    {
+        .name = MIN_WIDTH_PROP,
+        .type = G_TYPE_UINT,
+        .offset = G_STRUCT_OFFSET(XfceNotifyDaemon, min_width),
+        .default_value.u = MIN_WIDTH_DEFAULT,
+    },
     {
         .name = EXPIRE_TIMEOUT_ENABLED_PROP,
         .type = G_TYPE_BOOLEAN,
@@ -737,7 +761,7 @@ xfce_notify_daemon_closed(XfceNotification *notification,
                                                   (guint)reason);
 }
 
-static gboolean
+static inline gboolean
 xfce_notify_daemon_is_window_position_free(XfceNotifyDaemon *xndaemon,
                                            gint monitor_num,
                                            GdkRectangle *position,
@@ -745,15 +769,10 @@ xfce_notify_daemon_is_window_position_free(XfceNotifyDaemon *xndaemon,
 {
     for (GList *l = xndaemon->reserved_rectangles[monitor_num]; l; l = l->next) {
         GdkRectangle *rectangle = l->data;
-
-        DBG("Overlaps with (x=%i, y=%i) ?", rectangle->x, rectangle->y);
-
         if (gdk_rectangle_intersect(rectangle, position, NULL)) {
-            DBG("Yes");
+            DBG("Overlaps with (x=%i, y=%i)", rectangle->x, rectangle->y);
             *overlap_rect = *rectangle;
             return FALSE;
-        } else {
-            DBG("No");
         }
     }
 
@@ -768,14 +787,9 @@ xfce_notify_daemon_place_notification_window(XfceNotifyDaemon *xndaemon,
     GtkWidget *widget = GTK_WIDGET(window);
     GdkScreen *screen;
     GtkAllocation allocation;
-    GdkRectangle geom, initial, widget_geom;
-    gint monitor_num;
     gint max_width = 0;
+    gint monitor_num;
     gboolean found = FALSE;
-    gboolean is_center = xndaemon->notify_location == XFCE_NOTIFY_POS_TOP_CENTER ||
-        xndaemon->notify_location == XFCE_NOTIFY_POS_BOTTOM_CENTER;
-    gboolean is_ltr = gtk_widget_get_direction(GTK_WIDGET(window)) != GTK_TEXT_DIR_RTL;
-    gboolean center_opposite = FALSE;
 
     if (gtk_widget_has_screen(widget)) {
         screen = gtk_widget_get_screen(widget);
@@ -787,194 +801,89 @@ xfce_notify_daemon_place_notification_window(XfceNotifyDaemon *xndaemon,
     gtk_widget_get_allocation(widget, &allocation);
     monitor_num = xfce_notify_daemon_get_monitor_index(gdk_screen_get_display(screen), monitor);
     g_return_if_fail(monitor_num >= 0);
-    geom = xndaemon->monitors_workarea[monitor_num];
+
+    xndaemon->reserved_rectangles[monitor_num] = g_list_remove(xndaemon->reserved_rectangles[monitor_num],
+                                                               xfce_notify_window_get_geometry(window));
+
+    gint workarea_width = xndaemon->monitors_workarea[monitor_num].width;
+    gint workarea_height = xndaemon->monitors_workarea[monitor_num].height;
+    DBG("workarea: %dx%d", workarea_width, workarea_height);
 
     DBG("placing window, allocation=%dx%d+%d+%d", allocation.width, allocation.height, allocation.x, allocation.y);
 
-    /* Set initial geometry */
-    initial.width = allocation.width;
-    initial.height = allocation.height;
+    // Set initial position.  We ignore the start position
+    // (top/bottom-left/right/center) and just pretend it's all top-left.
+    // Later on we'll adjust the coordinates based on the actual start
+    // position.  The benefit here is that we don't need two different
+    // positioning algorithms for X11 and Wayland.  The downside is that if the
+    // start position is changed while notifications are displayed on the
+    // screen, future notifications could get placed on top of old ones.
+    gint xinitial = SPACE;
+    gint yinitial = SPACE;
 
-    switch(xndaemon->notify_location) {
-        case XFCE_NOTIFY_POS_TOP_LEFT:
-            initial.x = geom.x + SPACE;
-            initial.y = geom.y + SPACE;
-            break;
-        case XFCE_NOTIFY_POS_BOTTOM_LEFT:
-            initial.x = geom.x + SPACE;
-            initial.y = geom.y + geom.height - allocation.height - SPACE;
-            break;
-        case XFCE_NOTIFY_POS_TOP_RIGHT:
-            initial.x = geom.x + geom.width - allocation.width - SPACE;
-            initial.y = geom.y + SPACE;
-            break;
-        case XFCE_NOTIFY_POS_BOTTOM_RIGHT:
-            initial.x = geom.x + geom.width - allocation.width - SPACE;
-            initial.y = geom.y + geom.height - allocation.height - SPACE;
-            break;
-        case XFCE_NOTIFY_POS_TOP_CENTER:
-            initial.x = geom.x + (geom.width / 2) - (allocation.width / 2);
-            initial.y = geom.y + SPACE;
-            break;
-        case XFCE_NOTIFY_POS_BOTTOM_CENTER:
-            initial.x = geom.x + (geom.width / 2) - (allocation.width / 2);
-            initial.y = geom.y + geom.height - allocation.height - SPACE;
-            break;
-        default:
-            g_warning("Invalid notify location: %d", xndaemon->notify_location);
-            return;
-    }
+    GdkRectangle widget_geom = {
+        .x = xinitial,
+        .y = yinitial,
+        .width = allocation.width,
+        .height = allocation.height,
+    };
 
-    widget_geom.x = initial.x;
-    widget_geom.y = initial.y;
-    widget_geom.width = initial.width;
-    widget_geom.height = initial.height;
+    gboolean is_center = xndaemon->notify_location == XFCE_NOTIFY_POS_TOP_CENTER
+        || xndaemon->notify_location == XFCE_NOTIFY_POS_BOTTOM_CENTER;
+    gboolean past_center = FALSE;
 
     while (!found) {
         GdkRectangle overlap_rect = { 0, };
 
+        if (is_center && !past_center && widget_geom.x + widget_geom.width > workarea_width / 2) {
+            // This is a little hack to essentially divide our placement space
+            // into two independent regions when we're doing initial-center
+            // positioning.  Otherwise the window will not know how to
+            // translate positions on the "second half" to the right location.
+            widget_geom.x = workarea_width / 2 + SPACE;
+            past_center = TRUE;
+        }
+
         DBG("Test if the candidate overlaps one of the existing notifications.");
         if (xfce_notify_daemon_is_window_position_free(xndaemon, monitor_num, &widget_geom, &overlap_rect)) {
             DBG("We found a correct position.");
             found = TRUE;
         } else {
-            if (overlap_rect.width > max_width) {
+            if (overlap_rect.x == widget_geom.x && overlap_rect.width > max_width) {
                 max_width = overlap_rect.width;
+                DBG("Update max_width=%d", max_width);
             }
 
             DBG("Try above/below the current candidate position.");
-            switch (xndaemon->notify_location) {
-                case XFCE_NOTIFY_POS_TOP_LEFT:
-                case XFCE_NOTIFY_POS_TOP_RIGHT:
-                case XFCE_NOTIFY_POS_TOP_CENTER:
-                    widget_geom.y = overlap_rect.y + overlap_rect.height + SPACE;
-                    break;
-                case XFCE_NOTIFY_POS_BOTTOM_LEFT:
-                case XFCE_NOTIFY_POS_BOTTOM_RIGHT:
-                case XFCE_NOTIFY_POS_BOTTOM_CENTER:
-                    widget_geom.y = overlap_rect.y - widget_geom.height - SPACE;
-                    break;
-            }
+            widget_geom.y = overlap_rect.y + overlap_rect.height + SPACE;
 
-            if (is_center) {
-                // Since we initially center on the monitor, notifications of different widths
-                // will also be centered on the monitor.  But we really want neat columns that
-                // are left- or right-justified, so we have to adjust the x coord here too.
-                if ((is_ltr && !center_opposite) || (!is_ltr && center_opposite)) {
-                    widget_geom.x = overlap_rect.x;
-                } else {
-                    widget_geom.x = overlap_rect.x + overlap_rect.width - widget_geom.width;
-                }
-            }
-
-            if (widget_geom.y < geom.y || widget_geom.y + widget_geom.height > geom.y + geom.height) {
-                DBG("We reached the top/bottom of the monitor");
-                switch (xndaemon->notify_location) {
-                    case XFCE_NOTIFY_POS_TOP_LEFT:
-                    case XFCE_NOTIFY_POS_BOTTOM_LEFT:
-                        widget_geom.x = widget_geom.x + max_width + SPACE;
-                        break;
-                    case XFCE_NOTIFY_POS_TOP_RIGHT:
-                    case XFCE_NOTIFY_POS_BOTTOM_RIGHT:
-                        widget_geom.x = widget_geom.x - max_width - SPACE;
-                        break;
-                    case XFCE_NOTIFY_POS_TOP_CENTER:
-                    case XFCE_NOTIFY_POS_BOTTOM_CENTER:
-                        // Whether we try a column to the left or right depends on whether
-                        // or not we've exhausted all the space on the initial side of the
-                        // monitor.
-                        if ((is_ltr && !center_opposite) || (!is_ltr && center_opposite)) {
-                            widget_geom.x = widget_geom.x + max_width + SPACE;
-                        } else {
-                            widget_geom.x = widget_geom.x - max_width - SPACE;
-                        }
-                        break;
-                }
-                widget_geom.y = initial.y;
+            if (widget_geom.y + widget_geom.height > workarea_height) {
+                DBG("We reached the top/bottom of the monitor (y=%d, height=%d, workheight=%d) (add %d)", widget_geom.y, widget_geom.height, workarea_height, max_width);
+                widget_geom.x = widget_geom.x + max_width + SPACE;
+                widget_geom.y = yinitial;
                 max_width = 0;
 
-                if (widget_geom.x < geom.x || widget_geom.x + widget_geom.width > geom.x + geom.width) {
-                    if (is_center && !center_opposite) {
-                        DBG("Center positioning; use other side of center");
-                        center_opposite = TRUE;
-                    } else {
-                        DBG("There was no free space.");
-                        found = TRUE;
-                    }
-                    widget_geom.x = initial.x;
-                    widget_geom.y = initial.y;
+                if (widget_geom.x + widget_geom.width > workarea_width) {
+                    DBG("There was no free space.");
+                    found = TRUE;  // TODO: place on another monitor?
+                    widget_geom.x = xinitial;
+                    widget_geom.y = yinitial;
                 }
             }
         }
     }
 
-    DBG("Move the notification to: x=%i, y=%i", widget_geom.x, widget_geom.y);
-    xfce_notify_window_set_geometry(XFCE_NOTIFY_WINDOW(widget), widget_geom);
-    xndaemon->reserved_rectangles[monitor_num] = g_list_prepend(xndaemon->reserved_rectangles[monitor_num],
-                                                                xfce_notify_window_get_geometry(XFCE_NOTIFY_WINDOW(widget)));
+    DBG("Initial notification geom: %dx%d+%d+%d", widget_geom.width, widget_geom.height, widget_geom.x, widget_geom.y);
+    xfce_notify_window_set_geometry(XFCE_NOTIFY_WINDOW(widget), &widget_geom, &xndaemon->monitors_workarea[monitor_num]);
+    xndaemon->reserved_rectangles[monitor_num] = g_list_append(xndaemon->reserved_rectangles[monitor_num],
+                                                               xfce_notify_window_get_geometry(XFCE_NOTIFY_WINDOW(widget)));
 }
 
 static void
-xfce_notify_daemon_window_size_allocate(GtkWidget *widget,
-                                        GtkAllocation *allocation,
-                                        gpointer user_data)
-{
-    XfceNotifyDaemon *xndaemon = user_data;
-    XfceNotifyWindow *window = XFCE_NOTIFY_WINDOW(widget);
-    GdkScreen *widget_screen;
-    GdkDisplay *display;
-    GdkMonitor *monitor;
-    gint monitor_num;
-    GdkRectangle old_geom, geom;
-    gint cur_x, cur_y;
-
-    DBG("Size allocate called for %d", xfce_notify_window_get_id(window));
-
-    gtk_widget_set_allocation(widget, allocation);
-
-    old_geom = *xfce_notify_window_get_geometry(window);
-
-#ifdef ENABLE_X11
-    if (GDK_IS_X11_DISPLAY(gdk_display_get_default())) {
-        gtk_window_get_position(GTK_WINDOW(widget), &cur_x, &cur_y);
-    } else
-#endif
-    {
-        cur_x = old_geom.x;
-        cur_y = old_geom.y;
-    }
-
-    widget_screen = gtk_widget_get_screen (widget);
-    display = gdk_screen_get_display(widget_screen);
-
-    monitor = xfce_notify_window_get_monitor(window);
-    monitor_num = xfce_notify_daemon_get_monitor_index(display, monitor);
-
-    DBG("We are on the monitor %i", monitor_num);
-
-    geom = xndaemon->monitors_workarea[monitor_num];
-
-    DBG("Workarea: (%i,%i), width: %i, height:%i",
-        geom.x, geom.y, geom.width, geom.height);
-
-    if(old_geom.width != 0 && old_geom.height != 0) {
-        /* Notification has already been placed previously. */
-        GdkRectangle geom_union;
-        gdk_rectangle_union(&old_geom, &geom, &geom_union);
-        if (allocation->width == old_geom.width
-            && allocation->height == old_geom.height
-            && gdk_rectangle_equal(&geom, &geom_union)
-            && old_geom.x == cur_x
-            && old_geom.y == cur_y)
-        {
-            /* No updates are necessary */
-            return;
-        } else {
-            xndaemon->reserved_rectangles[monitor_num] = g_list_remove(xndaemon->reserved_rectangles[monitor_num],
-                                                                       xfce_notify_window_get_geometry(window));
-        }
-    }
+xfce_notify_daemon_window_need_position(XfceNotifyWindow *window, XfceNotifyDaemon *xndaemon) {
+    DBG("need-position for %d", xfce_notify_window_get_id(window));
 
+    GdkMonitor *monitor = xfce_notify_window_get_monitor(window);
     xfce_notify_daemon_place_notification_window(xndaemon, window, monitor);
 }
 
@@ -1214,6 +1123,49 @@ xfce_notify_log_insert(XfceNotifyLog *log,
     return id;
 }
 
+static gboolean
+recheck_window_monitor(gpointer data) {
+    MonitorCheckData *mcdata = data;
+    GtkWidget *widget = GTK_WIDGET(mcdata->window);
+    GdkDisplay *display = gtk_widget_get_display(widget);
+    GdkWindow *window = gtk_widget_get_window(widget);
+    GdkMonitor *monitor = gdk_display_get_monitor_at_window(display, window);
+
+    if (monitor == NULL) {
+        DBG("couldn't find monitor for window at all; using 0th");
+        monitor = gdk_display_get_monitor(display, 0);
+    }
+
+    DBG("found monitor index=%d for window %p", xfce_notify_daemon_get_monitor_index(display, monitor), mcdata->window);
+    xfce_notify_window_update_monitor(mcdata->window, monitor);
+
+    g_signal_connect(mcdata->window, "need-position",
+                     G_CALLBACK(xfce_notify_daemon_window_need_position), mcdata->xndaemon);
+    xfce_notify_daemon_place_notification_window(mcdata->xndaemon, mcdata->window, monitor);
+
+    g_object_unref(mcdata->window);
+    g_object_unref(mcdata->xndaemon);
+    g_free(mcdata);
+
+    return FALSE;
+}
+
+static gboolean
+xfce_notify_daemon_window_mapped(GtkWidget *widget, GdkEventAny *event, XfceNotifyDaemon *xndaemon) {
+    MonitorCheckData *mcdata = g_new0(MonitorCheckData, 1);
+    mcdata->xndaemon = g_object_ref(xndaemon);
+    mcdata->window = g_object_ref(XFCE_NOTIFY_WINDOW(widget));
+
+    // The monitor at this point will likely still not be correct, but if we
+    // wait *just* a little bit longer (for the wl_surface.enter() on Wayland),
+    // it should hopefully then be correct.
+    g_timeout_add(100, recheck_window_monitor, mcdata);
+
+    g_signal_handlers_disconnect_by_func(widget, xfce_notify_daemon_window_mapped, xndaemon);
+
+    return FALSE;
+}
+
 static gboolean
 notify_notify(XfceNotifyFdoGBus *skeleton,
               GDBusMethodInvocation   *invocation,
@@ -1600,12 +1552,23 @@ notify_notify(XfceNotifyFdoGBus *skeleton,
 
         switch (xndaemon->show_notifications_on) {
             case XFCE_NOTIFY_SHOW_ON_ACTIVE_MONITOR: {
-                GdkSeat *seat = gdk_display_get_default_seat(display);
-                GdkDevice *pointer = gdk_seat_get_pointer(seat);
-                gint x, y;
+#ifdef ENABLE_WAYLAND
+                if (GDK_IS_WAYLAND_DISPLAY(display) && gdk_display_get_n_monitors(display) != 1) {
+                    // On wayland we cannot determine the location of the
+                    // pointer, so we'll hope that the compositor will map the
+                    // window on the active monitor if we don't specify a
+                    // monitor
+                } else
+#endif
+                {
+                    GdkSeat *seat = gdk_display_get_default_seat(display);
+                    GdkDevice *pointer = gdk_seat_get_pointer(seat);
+                    gint x, y;
+
+                    gdk_device_get_position(pointer, NULL, &x, &y);
+                    monitors = g_list_append(monitors, gdk_display_get_monitor_at_point(display, x, y));
+                }
 
-                gdk_device_get_position(pointer, NULL, &x, &y);
-                monitors = g_list_append(monitors, gdk_display_get_monitor_at_point(display, x, y));
                 break;
             }
 
@@ -1639,17 +1602,29 @@ notify_notify(XfceNotifyFdoGBus *skeleton,
             GtkWidget *window = GTK_WIDGET(l->data);
             GtkRequisition nat_size;
             GtkAllocation allocation =  { 0, };
+            GdkMonitor *monitor;
 
-            g_signal_connect(window, "size-allocate",
-                             G_CALLBACK(xfce_notify_daemon_window_size_allocate), xndaemon);
             notify_update_theme_for_window(xndaemon, window, FALSE);
 
+            if (xndaemon->min_width_enabled) {
+                gtk_widget_set_size_request(window, xndaemon->min_width, -1);
+            }
             gtk_widget_get_preferred_size(window, NULL, &nat_size);
             DBG("preferred natural size: %dx%d", nat_size.width, nat_size.height);
             allocation.width = nat_size.width;
             allocation.height = nat_size.height;
             gtk_widget_set_allocation(window, &allocation);
-            xfce_notify_daemon_place_notification_window(xndaemon, XFCE_NOTIFY_WINDOW(window), xfce_notify_window_get_monitor(XFCE_NOTIFY_WINDOW(window)));
+
+            monitor = xfce_notify_window_get_monitor(XFCE_NOTIFY_WINDOW(window));
+            if (monitor != NULL) {
+                g_signal_connect(window, "need-position",
+                                 G_CALLBACK(xfce_notify_daemon_window_need_position), xndaemon);
+                xfce_notify_daemon_place_notification_window(xndaemon, XFCE_NOTIFY_WINDOW(window), monitor);
+            } else {
+                gtk_widget_add_events(GTK_WIDGET(window), GDK_STRUCTURE_MASK);
+                g_signal_connect_after(window, "map-event",
+                                       G_CALLBACK(xfce_notify_daemon_window_mapped), xndaemon);
+            }
         }
         xfce_notification_realize(notification);
         g_idle_add(notify_show_windows, notification);
@@ -1881,31 +1856,31 @@ xfce_notify_daemon_settings_changed(XfconfChannel *channel,
                 guint8 *loc = ((guint8 *)xndaemon) + settings[i].offset;
                 switch (settings[i].type) {
                     case G_TYPE_INT:
-                        *(gint *)loc = G_VALUE_TYPE(value) == settings[i].type
+                        *(gint *)(gpointer)loc = G_VALUE_TYPE(value) == settings[i].type
                             ? g_value_get_int(value)
                             : settings[i].default_value.i;
                         break;
 
                     case G_TYPE_UINT:
-                        *(guint *)loc = G_VALUE_TYPE(value) == settings[i].type
+                        *(guint *)(gpointer)loc = G_VALUE_TYPE(value) == settings[i].type
                             ? g_value_get_uint(value)
                             : settings[i].default_value.u;
                         break;
 
                     case G_TYPE_DOUBLE:
-                        *(gdouble *)loc = G_VALUE_TYPE(value) == settings[i].type
+                        *(gdouble *)(gpointer)loc = G_VALUE_TYPE(value) == settings[i].type
                             ? g_value_get_double(value)
                             : settings[i].default_value.d;
                         break;
 
                     case G_TYPE_BOOLEAN:
-                        *(gboolean *)loc = G_VALUE_TYPE(value) == settings[i].type
+                        *(gboolean *)(gpointer)loc = G_VALUE_TYPE(value) == settings[i].type
                             ? g_value_get_boolean(value)
                             : settings[i].default_value.b;
                         break;
 
                     case G_TYPE_STRING:
-                        *(gchar **)loc = g_strdup(
+                        *(gchar **)(gpointer)loc = g_strdup(
                             G_VALUE_TYPE(value) == settings[i].type
                                 ? g_value_get_string(value)
                                 : settings[i].default_value.s
@@ -1915,7 +1890,7 @@ xfce_notify_daemon_settings_changed(XfconfChannel *channel,
                     default:
                         if (g_type_is_a(settings[i].type, G_TYPE_ENUM)) {
                             const gchar *nick = G_VALUE_HOLDS_STRING(value) ? g_value_get_string(value) : NULL;
-                            *(gint *)loc = xfce_notify_enum_value_from_nick(settings[i].type, nick, settings[i].default_value.i);
+                            *(gint *)(gpointer)loc = xfce_notify_enum_value_from_nick(settings[i].type, nick, settings[i].default_value.i);
                         } else {
                             g_critical("Unhandled property type %s for property %s", g_type_name(settings[i].type), settings[i].name);
                         }
@@ -2014,31 +1989,31 @@ xfce_notify_daemon_load_config (XfceNotifyDaemon *xndaemon,
         guint8 *loc = ((guint8 *)xndaemon) + settings[i].offset;
         switch (settings[i].type) {
             case G_TYPE_INT:
-                *(gint *)loc = xfconf_channel_get_int(xndaemon->settings,
+                *(gint *)(gpointer)loc = xfconf_channel_get_int(xndaemon->settings,
                                                       settings[i].name,
                                                       settings[i].default_value.i);
                 break;
 
             case G_TYPE_UINT:
-                *(guint *)loc = xfconf_channel_get_uint(xndaemon->settings,
+                *(guint *)(gpointer)loc = xfconf_channel_get_uint(xndaemon->settings,
                                                         settings[i].name,
                                                         settings[i].default_value.u);
                 break;
 
             case G_TYPE_DOUBLE:
-                *(gdouble *)loc = xfconf_channel_get_double(xndaemon->settings,
+                *(gdouble *)(gpointer)loc = xfconf_channel_get_double(xndaemon->settings,
                                                             settings[i].name,
                                                             settings[i].default_value.d);
                 break;
 
             case G_TYPE_BOOLEAN:
-                *(gboolean *)loc = xfconf_channel_get_bool(xndaemon->settings,
+                *(gboolean *)(gpointer)loc = xfconf_channel_get_bool(xndaemon->settings,
                                                            settings[i].name,
                                                            settings[i].default_value.b);
                 break;
 
             case G_TYPE_STRING:
-                *(gchar **)loc = g_strdup(xfconf_channel_get_string(xndaemon->settings,
+                *(gchar **)(gpointer)loc = g_strdup(xfconf_channel_get_string(xndaemon->settings,
                                                                     settings[i].name,
                                                                     settings[i].default_value.s));
                 break;
@@ -2046,7 +2021,7 @@ xfce_notify_daemon_load_config (XfceNotifyDaemon *xndaemon,
             default:
                 if (g_type_is_a(settings[i].type, G_TYPE_ENUM)) {
                     const gchar *nick = xfconf_channel_get_string(xndaemon->settings, settings[i].name, NULL);
-                    *(gint *)loc = xfce_notify_enum_value_from_nick(settings[i].type, nick, settings[i].default_value.i);
+                    *(gint *)(gpointer)loc = xfce_notify_enum_value_from_nick(settings[i].type, nick, settings[i].default_value.i);
                 } else {
                     g_critical("Unhandled property type %s for property %s", g_type_name(settings[i].type), settings[i].name);
                 }
